package net.jansensan.loaders
{
	// + ----------------------------------------
	//		[ IMPORTS ]
	// + ----------------------------------------
	
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	
	
	/**
	 * <p>The XMLLoader class uses flash.net.URLLoader to specifically handle the loading of XML files.</p>
	 * 
	 * <b>Usage:</b>
	 * <p>
	 * function loadXML():void
	 * {
	 * var xmlLoader:XMLLoader = new XMLLoader();<br/>
	 * xmlLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);<br/>
	 * xmlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityHandler);<br/>
	 * xmlLoader.addEventListener(XMLLoader.READY, readyHandler);<br/>
	 * xmlLoader.load("someXMLFile.xml");<br/>
	 * }
	 * </p>
	 * function readyHandler():void
	 * {
	 * trace(xmlLoader.styleSheet);<br/>
	 * }
	 * </p>
	 * 
	 * @see flash.net.URLLoader
	 * @author Mat Janson Blanchet
	 */
	public class XMLLoader extends EventDispatcher
	{
		// + ----------------------------------------
		//		[ CONSTANTS ]
		// + ----------------------------------------
		
		// states
		
		
		// events
		public	static	const	READY	:String = "READY";
		
		// private
		
		
		// static
		
		
		// + ----------------------------------------
		//		[ VARIABLES ]
		// + ----------------------------------------
		
		// display objects
		
		
		// public
		
		
		// private / protected
		private	var	_URLLoader		:URLLoader;
		private	var	_isLoading		:Boolean = false;
		private	var	_XMLURL			:String;
		private	var	_XMLData		:XML;
		
		// static
		
		
		// + ----------------------------------------
		//		[ GETTERS / SETTERS ]
		// + ----------------------------------------
		
		public function get XMLData():XML
		{
			return _XMLData;
		}
		
		
		// + ----------------------------------------
		//		[CONSTRUCTOR ]
		// + ----------------------------------------
		
		public function XMLLoader()
		{
			
		}
		
		
		// + ----------------------------------------
		//		[ PUBLIC METHODS ]
		// + ----------------------------------------
		
		/**
		 * Initiates the loading of an XML file.
		 * @param XMLURL The path to the file, in string format. The XMLLoader will handle the conversion to URLRequest.
		 */
		public function load(XMLURL:String):void
		{
			//trace(this, "--- load ---");
			//trace("XMLURL: " + XMLURL);
			
			_XMLURL = XMLURL;
			_URLLoader = new URLLoader();
			addURLLoaderListeners();
			_URLLoader.load(new URLRequest(_XMLURL));
			_isLoading = true;
		}
		
		
		/**
		 * Cancels the loading of the file.
		 */
		public function abortLoading():void
		{
			removeURLLoaderListeners();
			try
			{
				_URLLoader.close();
			}
			catch(error:Error)
			{
				trace("\n", this, "--- abortLoading ---");
				trace(error.getStackTrace());
			}
			_isLoading = false;
		}
		
		
		// + ----------------------------------------
		//		[ PRIVATE/PROTECTED METHODS ]
		// + ----------------------------------------
		
		private function addURLLoaderListeners():void
		{
			_URLLoader.addEventListener(IOErrorEvent.IO_ERROR, urlLoaderIOErrorHandler, false, 0, true);
			_URLLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, urlLoaderSecurityErrorHandler, false, 0, true);
			_URLLoader.addEventListener(Event.COMPLETE, urlLoaderCompleteHandler, false, 0, true);
		}
		
		
		private function removeURLLoaderListeners():void
		{
			_URLLoader.removeEventListener(IOErrorEvent.IO_ERROR, urlLoaderIOErrorHandler);
			_URLLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, urlLoaderSecurityErrorHandler);
			_URLLoader.removeEventListener(Event.COMPLETE, urlLoaderCompleteHandler);
		}
		
		
		// + ----------------------------------------
		//		[ EVENT HANDLERS ]
		// + ----------------------------------------
		
		private function urlLoaderIOErrorHandler(event:IOErrorEvent):void
		{
			trace(this, "--- urlLoaderIOErrorHandler ---");
			trace("_XMLURL: " + _XMLURL);
			
			_isLoading = false;
			removeURLLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function urlLoaderSecurityErrorHandler(event:SecurityErrorEvent):void
		{
			trace(this, "--- urlLoaderSecurityErrorHandler ---");
			trace("_XMLURL: " + _XMLURL);
			
			_isLoading = false;
			removeURLLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function urlLoaderCompleteHandler(event:Event):void
		{
//			trace(this, "--- urlLoaderCompleteHandler ---");
			
			_isLoading = false;
			removeURLLoaderListeners();
			_XMLData = XML(_URLLoader.data);
			dispatchEvent(new Event(XMLLoader.READY));
		}
		
		
	}
}