package net.jansensan.robotlegstest.model
{
	// + ----------------------------------------
	//		[ IMPORTS ]
	// + ----------------------------------------
	
	import net.jansensan.robotlegstest.events.RobotLegsTestEvent;
	
	import flash.events.Event;
	
	import net.jansensan.loaders.AuthoringAssetsLoader;
	import net.jansensan.loaders.CSSLoader;
	
	import org.robotlegs.mvcs.Actor;
	
	
	/**
	 * @author Mat Janson Blanchet
	 */
	public class AssetsModel extends Actor
	{
		// + ----------------------------------------
		//		[ VARIABLES ]
		// + ----------------------------------------
		
		[Inject]
		public	var	configModel				:ConfigModel;
		
		private	var	_loaders				:Array;
		private	var	_loadedRatio			:Number;
		private	var	_authoringAssetsLoader	:AuthoringAssetsLoader;
		private	var	_CSSLoader				:CSSLoader;
		
		
		// + ----------------------------------------
		//		[CONSTRUCTOR ]
		// + ----------------------------------------
		
		public function AssetsModel()
		{
			super();
		}
		
		
		// + ----------------------------------------
		//		[ PUBLIC METHODS ]
		// + ----------------------------------------
		
		public function loadAssets():void
		{
			_loaders = [];
			
			_authoringAssetsLoader = new AuthoringAssetsLoader();
			_authoringAssetsLoader.addEventListener(AuthoringAssetsLoader.PROGRESS, assetsProgressHandler, false, 0, true);
			_authoringAssetsLoader.addEventListener(AuthoringAssetsLoader.READY, authoringAssetsProgressHandler, false, 0, true);
			_loaders.push(_authoringAssetsLoader);
			
			_CSSLoader = new CSSLoader();
			_CSSLoader.addEventListener(CSSLoader.PROGRESS, assetsProgressHandler, false, 0, true);
			_CSSLoader.addEventListener(CSSLoader.READY, CSSProgressHandler, false, 0, true);
			_loaders.push(_CSSLoader);
			
			_authoringAssetsLoader.load(configModel.authoringAssetsSWFURL);
			_CSSLoader.load(configModel.CSSURL);
		}
		
		
		// + ----------------------------------------
		//		[ PRIVATE/PROTECTED METHODS ]
		// + ----------------------------------------
		
		private function checkLoadProgress():void
		{
			var totalRatio:Number = 0;
			var i:int;
			var numLoops:int = _loaders.length;
			for(i = 0; i < numLoops; ++i)
			{
				totalRatio += _loaders[i].loadedRatio;
			}
			_loadedRatio = totalRatio / numLoops;
			
			var event:RobotLegsTestEvent;
			
			// progress
			if(_loadedRatio < 1)
			{
				event = new RobotLegsTestEvent(RobotLegsTestEvent.ASSETS_LOAD_PROGRESS);
			}
			// complete
			else
			{
				event = new RobotLegsTestEvent(RobotLegsTestEvent.ASSETS_LOAD_PROGRESS);
				dispatch(event);
				event = new RobotLegsTestEvent(RobotLegsTestEvent.ASSETS_LOAD_COMPLETE);
			}
			dispatch(event);
		}
		
		
		// + ----------------------------------------
		//		[ EVENT HANDLERS ]
		// + ----------------------------------------
		
		private function assetsProgressHandler(event:Event):void
		{
			checkLoadProgress();
		}
		
		
		private function authoringAssetsProgressHandler(event:Event):void
		{
			_authoringAssetsLoader.addEventListener(AuthoringAssetsLoader.PROGRESS, assetsProgressHandler);
			_authoringAssetsLoader.addEventListener(AuthoringAssetsLoader.READY, authoringAssetsProgressHandler);
			checkLoadProgress();
		}
		
		
		private function CSSProgressHandler(event:Event):void
		{
			_CSSLoader.addEventListener(CSSLoader.PROGRESS, assetsProgressHandler);
			_CSSLoader.addEventListener(CSSLoader.READY, CSSProgressHandler);
			checkLoadProgress();
		}
		
		
		// + ----------------------------------------
		//		[ GETTERS / SETTERS ]
		// + ----------------------------------------
		
		public function get loadedRatio():Number
		{
			return _loadedRatio;
		}
		
		
	}
}